/*****************************************************************
 * Dummyload projekt 2023 - 24 OZ7SKB Skanderborg EDR
 * 
 * Rev. 1.1 16052024  OZ4KAJ
 * 
 * Inspireret af W8TEE Jack Purdum og W6DQ Dennis Kidder
 * Arduino Projects for Amateur Radio 2015
 * 
 * !!!!!!!!!!!!!!!!! ADVARSEL !!!!!!!!!!!!!!!!!
 * Husk at koden skal kompileres og uploades til Arduino Uno, 
 * hvor kun Arduino Uno er tilkoblet. 
 * Alså uden display og board med elektronik tilkoblet.
 * RISIKO: Arduino Uno kan blive ødelagt med kortslutning af AREF-porten
 *****************************************************************/
 
// Inkluder hjælpelibrary
#include <LiquidCrystal.h>

// Konstanter og variabler defineres

#define DEBUG                   // Udmarkers med // til drift

#define ZEROAJUST          10   // Det trækkes fra analogInBit så beregning har udgangspunkt i nul
#define DEFAULTPRECISION    0	  // Decimal placering
#define MAXANALOGVALUE   1000   // Validt input 0 - 1000
#define VALUEWIDTH          8   // Definere resultatfelt med '-', '.'
#define DIODEVOLTAGEDROP  1.6   // To 1N4814...måske en anden værdi ved andre dioder
#define DISPLAYDELAY     1000   // Pauser display i 1 sekund. Forhindre flimmer.

#define UPDATEDELAY       100   // Pause til aflæsning af A1  
#define ERRORDELAY        400   // Pause til visning af fejl
#define STARTUPDELAY     1000   // Pause ved opstart til visning af version mm
                                // Tekst er fri, max 2 x 16


const int LCDNUMCOLS =     16;  // Antal kolonner på display
const int LCDNUMROWS =      2;  // Antal linier på display
const int DLSENSORPIN =    A1;  // Definere analog port A1 som input

char buffer[LCDNUMCOLS + 1];    // Buffer til output
float ScaleFactor =       10;       // Skalerinsafaktor for Volt. Passer til 100W
const float MESASUREDDUMMYLOADRESISTANCE = 50.4;  // Modstanden i dummyload

// Definere LCD pins
const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
 
/*********** Initialisering af Uno ***********/
void setup()
{
  Serial.begin(9600);             // Initiere seriel kommunikation

  analogReference(EXTERNAL);      // Brug den eksterne spaendingsreference
  lcd.begin(16, 2);               // Initiere display til 16 x 2

  #ifdef DEBUG                    
    Serial.println("---------------------------------------------------------------------------");
    Serial.println("OZ7SKB Dummyload - Version 1.1 2024");
  #endif

  // Startup visning på display. Kan frit ændres. Max 16 x 2 tegn
  lcd.print("OZ4KAJ Dummyload");    // Viser teksten
  lcd.setCursor(0, 1);              // Linje 2
  lcd.print("Version 1.0 2024");    // Viser teksten
  delay(STARTUPDELAY);              // Hold en pause
  lcd.clear();                      // Display renses  
} // setup() slut

/************ Arbejdsprogrammet gentages uendeligt *********/
void loop()
{
  // Lokale variable
  int analogInBit;
  float val;

  analogInBit = analogRead(DLSENSORPIN);      // Aflæs input
  analogInBit = analogInBit - ZEROAJUST;      // Justering i forhold til nul

  #ifdef DEBUG  
    Serial.println("---------------------------------------------------------------------------");
    Serial.print("analogInBit = ");
    Serial.println(analogInBit);
  #endif

  if (analogInBit) {
    BuildTheOutput(analogInBit);              
    ShowTheOutput(analogInBit, val); 
    delay(DISPLAYDELAY);
  }
} //loop() slut

/************ BeregnOutput ***********0***0***
 *  Funktionen beregner den målte effekt
 *  og formattere resultatet til LCD
 *********************************************/
void BuildTheOutput(int voltsIn)
{
  int i;
  float scaledVolts;
  float RMS;
  float watts;

  scaledVolts = (float (voltsIn)) + DIODEVOLTAGEDROP;    // Justerer for diode spændingsfald

  #ifdef DEBUG  
   Serial.print("scaledVolts1 = ");
   Serial.println(scaledVolts);
  #endif

  scaledVolts = scaledVolts / ScaleFactor;               // Skalerer til visning

  #ifdef DEBUG  
   Serial.print("scaledVolts2 = ");
   Serial.println(scaledVolts);
  #endif

  RMS = scaledVolts / 1.414;                             // Beregn RMS ud fra volt

  #ifdef DEBUG  
   Serial.print("RMS = ");
   Serial.println(RMS);
  #endif

  watts = (RMS * RMS) / MESASUREDDUMMYLOADRESISTANCE;    // Beregn watt

  #ifdef DEBUG                    
    Serial.print("watts = ");
    Serial.println(watts);
  #endif

  dtostrf(watts, VALUEWIDTH, DEFAULTPRECISION, buffer);  // Formattering af resultatet
  strcat(buffer, " watts");
} // BuildTheOutput() slut


/************ VisOutput *********************0*
 *  Funktionen viser resultatet på LCD
 **********************************************/
 void ShowTheOutput(int analogIn, float watts)
{
  lcd.setCursor(0,0);                  // Placerer cursor i 0,0 på display
  lcd.print(buffer);

  // Her kan visning i anden dilplaylinje skrives

  if (analogIn >= MAXANALOGVALUE)    // Er værdien tilladt....
  {                             // ...hvis ikke, så gør opmærksom på det.   
    lcd.setCursor(0, 1);
    lcd.print("   Overrange!   ");  
    lcd.noDisplay();                   // Blink beskeden
    lcd.clear();                      // Display renses  
    delay(ERRORDELAY);
    lcd.print("   Overrange!   ");  
    lcd.display();
    delay(ERRORDELAY); 
  }        
  else 
  { 
    // delay(UPDATEDELAY); 
    } 

} // ShowTheOutput() slut
 
